@@ -143,7 +143,7 @@ class GregorianTime(object):
         year += steps
         dt = datetime(year=year, month=1, day=1)
 
-    return dt.strftime( self.FORMATS[self._step] )
+    return int(dt.strftime( self.FORMATS[self._step] ))
 
   def from_bucket(self, bucket):
     '''
@@ -152,6 +152,7 @@ class GregorianTime(object):
     # NOTE: this is due to a bug somewhere in strptime that does not process
     # the week number of '%Y%U' correctly. That bug could be very specific to
     # the combination of python and ubuntu that I was testing.
+    bucket = str(bucket)
     if self._step == 'weekly':
       year, week = bucket[:4], bucket[4:]
       normal = datetime(year=int(year), month=1, day=1) + timedelta(weeks=int(week))
@@ -172,6 +173,8 @@ class GregorianTime(object):
       bucket = self.to_bucket(start, step)
       bucket_time = self.from_bucket( bucket )
       if bucket_time >= end:
+        if bucket_time==end:
+          rval.append( bucket )
         break
       rval.append( bucket )
       step += 1
@@ -313,6 +316,24 @@ class Timeseries(object):
       config['expire'] = interval_calc.ttl( steps )
       config['coarse'] = (resolution==step)
 
+  def list(self):
+    '''
+    List all of the stat names that are stored.
+    '''
+    raise NotImplementedError()
+
+  def properties(self, name):
+    '''
+    Get the properties of a stat.
+    '''
+    raise NotImplementedError()
+
+  def expire(self, name):
+    '''
+    Manually expire data for storage engines that do not support auto expiry.
+    '''
+    raise NotImplementedError()
+
   def insert(self, name, value, timestamp=None, intervals=0):
     '''
     Insert a value for the timeseries "name". For each interval in the 
@@ -355,6 +376,32 @@ class Timeseries(object):
     '''
     raise NotImplementedError()
 
+  def iterate(self, name, interval, **kwargs):
+    '''
+    Returns a generator that iterates over all the intervals and returns
+    data for various timestamps, in the form:
+
+      ( unix_timestamp, data )
+
+    This will check for all timestamp buckets that might exist between
+    the first and last timestamp in a series. Each timestamp bucket will
+    be fetched separately to keep this memory efficient, at the cost of
+    extra trips to the data store.
+
+    Keyword arguments are the same as get().
+    '''
+    config = self._intervals.get(interval)
+    if not config:
+      raise UnknownInterval(interval)
+    properties = self.properties(name)[interval]
+
+    i_buckets = config['i_calc'].buckets(properties['first'], properties['last'])
+    for i_bucket in i_buckets:
+      data = self.get(name, interval,
+        timestamp=config['i_calc'].from_bucket(i_bucket), **kwargs)
+      for timestamp,row in data.items():
+        yield (timestamp,row)
+
   def get(self, name, interval, **kwargs):
     '''
     Get the set of values for a named timeseries and interval. If timestamp
@@ -457,20 +504,20 @@ class Timeseries(object):
 
     # Fugly range determination, all to get ourselves a start and end 
     # timestamp. Adjust steps argument to include the anchoring date.
-    if not end:
-      if start:
-        start_bucket = config['i_calc'].to_bucket( start )
-        end_bucket = config['i_calc'].to_bucket( start, steps-1 )
-      else:
+    if end is None:
+      if start is None:
         end = time.time()
         end_bucket = config['i_calc'].to_bucket( end )
         start_bucket = config['i_calc'].to_bucket( end, (-steps+1) )
+      else:
+        start_bucket = config['i_calc'].to_bucket( start )
+        end_bucket = config['i_calc'].to_bucket( start, steps-1 )
     else:
       end_bucket = config['i_calc'].to_bucket( end )
-      if start:
-        start_bucket = config['i_calc'].to_bucket( start )
-      else:
+      if start is None:
         start_bucket = config['i_calc'].to_bucket( end, (-steps+1) )
+      else:
+        start_bucket = config['i_calc'].to_bucket( start )
       
     # Now that we have start and end buckets, convert them back to normalized
     # time stamps and then back to buckets. :)
